import re
from django.core.management.base import BaseCommand
from django.core.serializers import serialize
from django.apps import apps
import os


class Command(BaseCommand):

    help = "Fixture Dumper. Export models to fixtures."
    format = 'json'
    indent = None
    export_path = 'fixtures'

    def add_arguments(self, parser):
        # Positional arguments
        parser.add_argument('app_name',
                            nargs='*',
                            help='Provide app names as is listed in INSTALLED_APPS. '
                                 'If no app_name provided, will dump everything',
                            type=str)
        parser.add_argument('--indent',
                            dest='indent',
                            type=int,
                            help='JSON indentation files.', )
        parser.add_argument('--exportpath',
                            dest='export_path',
                            type=str,
                            help='Export path. Default fixtures/', )

    def get_json_filename(self, name):
        s1 = re.sub('(.)([A-Z][a-z]+)', r'\1-\2', name)
        return "{0}.json".format(re.sub('([a-z0-9])([A-Z])', r'\1-\2', s1).lower())

    def get_app_names(self):
        return [app.label for app in apps.get_app_configs() if app.label != "contenttypes"]

    def get_models(self, app_name):
        return apps.get_app_config(app_name).get_models()

    def dump(self, model, app_name, to_file):
        self.stdout.write("\t{0}:".format(model.__name__))
        self.stdout.write("\t- Serializing {0} objects...".format(model.objects.all().count()), ending="")
        self.stdout.flush()
        serialized_data = serialize(self.format, model.objects.all(), indent=self.indent)
        self.stdout.write(" [ OK ]")
        self.stdout.write("\t- Dumping in {0}/{1}/{2}...".format(self.export_path, app_name, to_file), ending="")
        self.stdout.flush()
        with open("{0}/{1}/{2}".format(self.export_path, app_name, to_file), 'w+') as fixture_file:
            fixture_file.write(serialized_data)
            self.stdout.write(" [ OK ]")

    def handle(self, *args, **options):
        if options["indent"]:
            self.indent = options["indent"]
        if options["export_path"]:
            self.export_path = options["export_path"]
        dump_app_names = options["app_name"] if options["app_name"] else self.get_app_names()
        for app_name in dump_app_names:
            self.stdout.write("{0}".format(app_name))
            app_models = self.get_models(app_name)
            for model in app_models:
                os.makedirs(os.path.dirname("{0}/{1}/".format(self.export_path, app_name)), exist_ok=True)
                self.dump(model, app_name, self.get_json_filename(model.__name__))



from distutils.core import setup
from setuptools import find_packages


setup(
    name='django_loader_dumper',
    packages=find_packages(),
    version='1.0.3',
    description='Commands for loading and dumping fixtures based on app names.',
    author='Andruten',
    author_email='andusaben@gmail.com',
    url='https://github.com/andruten/django_loader_dumper',
    download_url='https://github.com/andruten/django_loader_dumper/archive/1.0.3.tar.gz',
    keywords=['django', 'fixtures', 'dumpdata', 'loaddata', 'commands', 'json'],
    classifiers=[],
)



from django.core.management.base import BaseCommand
from django.core.management import call_command
from django.db.utils import IntegrityError
from django.apps import apps
from glob import glob


class Command(BaseCommand):
    help = "Fixture loader. Imports models from fixtures."
    export_path = "fixtures"

    def add_arguments(self, parser):
        # Positional arguments
        parser.add_argument('app_name',
                            nargs='*',
                            help='Provide at least one app name as is listed in INSTALLED_APPS',
                            type=str)
        parser.add_argument('--exportpath',
                            dest='export_path',
                            type=str,
                            help='Export path. Default fixtures/', )

    def get_fixtures(self, app_name):
        fixtures = []
        for fixture_name in glob("{0}/{1}/*.json".format(self.export_path, app_name)):
            fixtures.append({
                "name": fixture_name,
                "applied": False,
                "retries": 0,
            })
        return fixtures

    def get_app_names(self):
        return [app.label for app in apps.get_app_configs()]

    def handle(self, *args, **options):
        if options["export_path"]:
            self.export_path = options["export_path"]
        if options["settings"] and ".production" in options["settings"]:
            fix = input("Fixtures will be imported in the PRODUCTION environment. Are you sure? [Y/n] ")
            if fix != "Y":
                self.stderr.write("Operation cancelled.")
                return False
        dump_app_names = options["app_name"] if options["app_name"] else self.get_app_names()
        for app_name in dump_app_names:
            fixtures = self.get_fixtures(app_name)
            while [fixture for fixture in fixtures if not fixture["applied"]]:
                for fixture in fixtures:
                    if fixture["applied"]:
                        continue
                    try:
                        self.stdout.write("Loading {0}...".format(fixture["name"]))
                        self.stdout.flush()
                        call_command("loaddata", fixture["name"])
                        fixture["applied"] = True
                        fixture["retries"] = 0
                    except IntegrityError as e:
                        self.stderr.write("[ FAILED ] Fixture " + fixture["name"] + ": " + e.args[0])
                        self.stdout.write("\tIt will be retried later...")
                        fixture["retries"] += 1
                if any(fixture["retries"] >= 5 for fixture in fixtures):
                    self.stderr.write("Couldn't load all fixtures.")
                    break
